#include <stdio.h>
#include <sys/time.h>
#include <usb.h>

void tune(usb_dev_handle *handle, int freq, int tv)
{
	int x;
	int i;
	unsigned char buf;

	if(tv)
		x = 0xce00;
	else
		x = 0x8806;

	if(freq < 170000)
		x |= 0xa0;
	else if(freq > 470000)
		x |= 0x30;
	else
		x |= 0x90;

	i = freq + 10700;

	if(tv)
		i *= 16;
	else
		i *= 20;

	i /= 1000;

	usb_control_msg(handle, 0x40, 0x01, i , x, NULL, 0, 1000);

	for(i = 0;i < 10;i++) {
		usleep(100000);
		buf = 0;
		usb_control_msg(handle, 0xc0, 0x01, 0, 0, &buf, 1, 1000);

		if(buf & 0x40)
			break;
	}

	if(!(buf & 0x40))
		printf("NOT LOCKED: f = %d kHz, tv = %d\n", freq, tv);
}

int main(int argc,char **argv)
{
	struct usb_bus *bus;
	struct usb_device *dev;
	usb_dev_handle *handle;
	int i;

	if(argc < 2)
		return 1;
	i = strtoul(argv[1], NULL, 0);

	usb_init();
	usb_find_busses();
	usb_find_devices();

	for(bus = usb_get_busses(); bus; bus = bus->next)
		for(dev = bus->devices; dev; dev = dev->next) {
			if((dev->descriptor.idVendor == 0x04b4) && (dev->descriptor.idProduct == 0x8614))
				goto found;
		}

	return 1;

found:
	handle = usb_open(dev);
	if(!handle)
		return 1;
	usb_claim_interface(handle, 0);

	tune(handle, 500000, 1);
	tune(handle, i, 1);
	tune(handle, i, 0);

out:
	usb_close(handle);

	return 0;
}

