#include <stdio.h>
#include <sys/time.h>
#include <usb.h>


// adr todev len [data....]
int main(int argc,char **argv)
{
	struct usb_bus *bus;
	struct usb_device *dev;
	usb_dev_handle *handle;
	unsigned long long int dat = 0;
	unsigned char adr, todev, len=0;
	int i;

	if(argc != 4 && argc != 5) {
		printf("not enough args\n");
		return 1;
	}

	adr = strtoul(argv[1], NULL, 0);
	todev = strtoul(argv[2], NULL, 0);
	len = strtoul(argv[3], NULL, 0);

	if(todev) {
		dat = strtoull(argv[4], NULL, 0);
		printf("%02x[%d]: %s 0x%016llx\n", adr, len, todev ? " [PC]==>[DEV] " : " [PC]<==[DEV] ", dat);
	}

	usb_init();
	usb_find_busses();
	usb_find_devices();

	for(bus = usb_get_busses(); bus; bus = bus->next)
		for(dev = bus->devices; dev; dev = dev->next) {
			if((dev->descriptor.idVendor == 0xfffe) && (dev->descriptor.idProduct == 0x0002))
				goto found;
		}

	return 1;

found:
	handle = usb_open(dev);
	if(!handle)
		return 1;
	usb_claim_interface(handle, 0);

	int index=0, value=0;
	//dev, int requesttype, int request, int value, int index, char *bytes, int size, int timeout
	if(todev) {
		switch(len) {
			case 8:
				index = ((dat >> 8) & 0xff) << 8 | (dat & 0xff);
				value = ((dat >> 24) & 0xff) << 8 | ((dat >> 16) & 0xff);
				printf("send[0] %04x | %04x\n", index, value);
				usb_control_msg(handle, 0x40, 0x04, value, index, NULL, 0, 1000);
				index = ((dat >> 40) & 0xff) << 8 | ((dat >> 32) & 0xff);
				value = ((dat >> 56) & 0xff) << 8 | ((dat >> 48) & 0xff);
				usb_control_msg(handle, 0x40, 0x03, value, index, NULL, 0, 1000);
				printf("send[1] %04x | %04x\n", index, value);
				break;
			case 6:
				value = ((dat >> 8) & 0xff) << 8 | (dat & 0xff);
				printf("send[0] %04x | %04x\n", index, value);
				usb_control_msg(handle, 0x40, 0x04, value, index, NULL, 0, 1000);
				index = ((dat >> 24) & 0xff) << 8 | ((dat >> 16) & 0xff);
				value = ((dat >> 40) & 0xff) << 8 | ((dat >> 32) & 0xff);
				usb_control_msg(handle, 0x40, 0x03, value, index, NULL, 0, 1000);
				printf("send[1] %04x | %04x\n", index, value);
				break;
			case 4:
				value = ((dat >> 24) & 0xff) << 8 | ((dat >> 16) & 0xff);
				index = ((dat >> 8) & 0xff) << 8 | (dat & 0xff);
				usb_control_msg(handle, 0x40, 0x03, value, index, NULL, 0, 1000);
				printf("send[1] %04x | %04x\n", index, value);
				break;
			case 2:
				value = ((dat >> 8) & 0xff) << 8 | (dat & 0xff);
				usb_control_msg(handle, 0x40, 0x03, value, index, NULL, 0, 1000);
				printf("send[1] %04x | %04x\n", index, value);
				break;
			default:
				printf("bad length, must be 2/4/6 or 8 bytes\n");
		}
	}

	  //spibuf[0] = wValueL;
	  //spibuf[1] = wValueH;
	  //spi_xfer(wIndexL, wIndexH, spibuf);
	index = len << 8 | adr;
	if(!todev) index |= 0x80;

	int usblen = usb_control_msg(handle, 0x40, 0x02, 0x00, index, NULL, 0, 1000000);
	if(usblen != 0x00)
		printf("bad len: %d\n", usblen);
	//else
	//	printf("ok\n");

	if(!todev) {
		unsigned char buf[8];
		usblen = usb_control_msg(handle, 0xc0, 0x02, 0, 0, buf, 8, 1000);
		//printf("%d\n", len);
		if(usblen < 0)
			goto out;
		printf("%02x.%02x.%02x.%02x.%02x.%02x.%02x.%02x\n",
			buf[0],buf[1],buf[2],buf[3],buf[4],buf[5],buf[6],buf[7]);
		for(i=0;i<len;i++)
			dat = dat << 8 | buf[i];
		switch(len) {
			case 2:
				printf("reg[%02x]:%d = 0x%04llx\n", adr, len, dat);
				break;
			case 4:
				printf("reg[%02x]:%d = 0x%08llx\n", adr, len, dat);
				break;
			case 6:
				printf("reg[%02x]:%d = 0x%012llx\n", adr, len, dat);
				break;
			case 8:
			default:
				printf("reg[%02x]:%d = 0x%016llx\n", adr, len, dat);
				break;
		}

	}
out:
	usb_close(handle);

	return 0;
}

