#include "spi.h"
#include "fx2regs.h"
#include "syncdelay.h"

#include "delay.h"

//TODO: use two-wire spi

#define SPI_CS   0x80 //FLAGD join I/O_RESET + !CS pins together.
#define SPI_SCK  0x08 //WU2
#define SPI_UP   0x02 //INT1 I/O_UPDATE
#define SPI_SDIO 0x01 //INT0 SDIO

//PORTA:
// 7 - FLAGD - I/O_RESET + !CS
// 6 - PKTEND
// 5 - FIFOADR1
// 4 - FIFOADR0
// 3 - WU2 - SCK
// 2 - SLOE
// 1 - INT1 - I/O_UPDATE
// 0 - INT0 - SDIO

void spi_init()
{
	IOA |= SPI_CS|SPI_SCK|SPI_SDIO;
	OEA |= SPI_CS|SPI_SCK|SPI_UP|SPI_SDIO;
}

#if 0
void spi_xfer(xdata unsigned char *buf, int len)
{
	int i,j;
	unsigned char x;

	IOA |= SPI_CS;
	SYNCDELAY;

	for(i = 0;i < len;i++) {
		x = buf[i];
		buf[i] = 0;

		for(j = 0;j < 8;j++) {
			if(x & (1 << j))
				IOA |= SPI_MOSI;
			else
				IOA &= ~SPI_MOSI;
			SYNCDELAY;

			IOA |= SPI_SCK;
			if(IOA & SPI_MISO)
				buf[i] |= 1 << j;
			SYNCDELAY;

			IOA &= ~SPI_SCK;
		}
	}

	IOA &= ~SPI_CS & ~SPI_MOSI;
}
#endif

void spi_write_byte(unsigned char byte)
{
	unsigned char mask = 0x80;
	do {
		IOA &= ~SPI_SCK; // SCK: \_
		SYNCDELAY;

		if(byte & mask)
			IOA |= SPI_SDIO;
		else
			IOA &= ~SPI_SDIO;
		SYNCDELAY;

		IOA |= SPI_SCK; // SCK: _/
		SYNCDELAY;
		mask = mask >> 1;
	} while(mask != 0);
}

void spi_read_byte(xdata unsigned char *buf)
{
	unsigned char mask = 0x80;
	do {
		IOA &= ~SPI_SCK; // SCK: \_
		SYNCDELAY;

		IOA |= SPI_SCK; // SCK: _/
		SYNCDELAY;

		if(IOA & SPI_SDIO)
			*buf |= mask;
		SYNCDELAY;
		mask = mask >> 1;
	} while(mask != 0);
}

void spi_xfer(unsigned char reg, int len, xdata unsigned char *buf)
{
	int i;
	IOA &= ~SPI_CS; SYNCDELAY; SYNCDELAY;
	spi_write_byte(reg);

	if(reg &  (1 << 7)) {
		OEA &= ~SPI_SDIO; //disable output for sdio
		for(i = 0;i < len;i++) {
			buf[i]=0;
			spi_read_byte(buf+i);
		}
	} else {
		for(i = 0;i < len;i++)
			spi_write_byte(buf[i]);

		//write from i/o buf to actual register
		IOA |= SPI_UP; SYNCDELAY; SYNCDELAY; SYNCDELAY; SYNCDELAY; //wait internal i/o update sysclk/4/4 = 16 cycles for 24mhz
		 mdelay(1);
		IOA &= ~SPI_UP;
		 mdelay(1);
	}

	IOA |= SPI_CS; SYNCDELAY; SYNCDELAY;

	if(reg &  (1 << 7)) {
		OEA |= SPI_SDIO; //enable output for sdio
		SYNCDELAY;
	}
}
